#include "mbed.h"
#include "SDFileSystem.h"    // SD file system header from handbook/cookbook offical mbed library
#include "MODSERIAL.h"   
#include "UM6_usart.h"     // UM6 USART HEADER
#include "UM6_config.h"    // UM6 CONFIG HEADER
#include "GPS.h"

//------------ system and interface setup ----------------------------//
MODSERIAL pc(USBTX, USBRX);  // sets up serial connection to pc terminal

//------------ Hardware setup ----------------------------------------//
DigitalOut pc_led(LED1);               // LED1 = PC SERIAL
DigitalOut uart_led(LED2);             // LED2 = UM6 SERIAL
DigitalOut log_led(LED3);              // debug LED
DigitalOut debug_led(LED4);            // debug LED
DigitalIn enable(p10);                 // enable logging pin
DigitalOut sync(p11);                  // sychronization (with CAN logger) pin  
SDFileSystem sd(p5, p6, p7, p8, "sd"); // the pinout on the mbed Cool Components workshop board
GPS gps(NC,p27); 

// interupt function for processing uart messages --------------------//
void rxCallback(MODSERIAL_IRQ_INFO *q) {
    if (um6_uart.rxBufferGetCount() >=  MAX_PACKET_DATA) {
        uart_led = !uart_led;  // Lights LED when uart RxBuff has > 40 bytes
        Process_um6_packet();
    }
}

//------------ LogData interrupt function ----------------------------//
int flag=0;
void LogData() {
            flag=1;     //interrupt sets flag that causes variables to be logged
} 

//============= Main Program =========================================//
int main() {
    Ticker tick;         
    Timer t;  
    GPS_Time q1;
    GPS_VTG  v1;
    
    pc.baud(115200);       // baud rate to pc interface
    um6_uart.baud(115200); // baud rate to um6 interface
    gps.baud(57600);
    gps.format(8, GPS::None, 1);
    
    //---- call interrupt functions --------------------------//
    um6_uart.attach(&rxCallback, MODSERIAL::RxIrq); // attach interupt function to uart
    tick.attach(&LogData, 2); // attaches LogData function to 'tick' ticker interrupt every 0.5s
    t.start(); // start logging time        

    //---------- setup sd card directory------------------------------// 
    int FileNo = 0;
    mkdir("/sd/log_data", 0777);  

    //---- main while loop -------------------------------------------// 
    while(1) {            
            //---- Setup file on SD card ----------------------------//
            printf("Opening sd card\n");           
            char buffer[50];
            sprintf(buffer, "/sd/log_data/%i.csv", FileNo);
            //****************************************************
            // This is necessary for card to work when reconnected
            // Initialise disk
            sd.disk_initialize();
            //****************************************************
            // Open a file for write
            FILE *fp = fopen(buffer, "w"); 
            if(fp == NULL) {
                error("Could not open file for write\n");
            }
            //--- print TEST signals to file, header
            //fprintf(fp,"time(s),Yaw(deg),Accel(m/s2),GPS Speed(m/s) \r");
            //--- print ALL signals to file, header
            fprintf(fp, "time(s),Yaw(deg),Roll(deg),Pitch(deg),GyroX(deg/s),GyroY(deg/s),GyroZ(deg/s),AccelX(g),AccelY(g),AccelZ(g),GPScourse(deg),GPSspeed(m/s),Latitude(deg),Longitude(deg) \r");   // sends header to file
            
            while (!pc.readable()) {  
                    if(flag==1) {  
                        log_led=1;  // turns on LED3 to indicate logging               
                        float time=t.read();
                        float Yaw=data.Yaw;
                        float Roll=data.Roll;
                        float Pitch=data.Pitch;
                        float GyroX=data.Gyro_Proc_X;
                        float GyroY=data.Gyro_Proc_Y;
                        float GyroZ=data.Gyro_Proc_Z;
                        float AccelX=data.Accel_Proc_X;
                        float AccelY=data.Accel_Proc_Y;
                        float AccelZ=data.Accel_Proc_Z;
                        double GPSlong=gps.longitude();        // currently not reading GPS longitude correctly
                        double GPSlat=gps.latitude();                   // currently not reading GPS latitude correctly
                        double GPSalt=gps.altitude(); 
                        float GPScourse=data.GPS_course;
                        float GPSspeed=data.GPS_speed;
                    
                        //----- print TEST signals----------------------------//
                       // fprintf(fp,"%.3f,%.2f,%.4f,%.2f \r",time,Yaw,AccelX,GPSspeed); 
                        pc.printf("time %.3f,Yaw %f, Lat %f, Long %f, Alt %f \n",time,Yaw,GPSlat,GPSlong,GPSalt);                       
                        gps.timeNow(&q1);
                        pc.printf("%c %02d:%02d:%02d %02d/%02d/%04d\r\n",q1.status, q1.hour, q1.minute, q1.second, q1.day, q1.month, q1.year);                               
                        gps.vtg(&v1);
                        pc.printf("Speed(kph):%lf,Track(true):%lf\r\n", v1.velocity_kph(), v1.track_true());
                        
                        //pc.printf("time %.3f,Yaw %f,Accel %f, Speed %f \n",time,Yaw,AccelX,GPSspeed);
                       // pc.printf("time %f,LongIn 0x%08X, LongOut %f\n",time,GPSlong,*(int *)&GPSlong);
        
                        //----- print ALL signals to file --------------------//  
                 //       fprintf(fp, "%3.3f, %3.1f,%3.1f,%3.1f, %3.2f,%3.2f,%3.2f, %3.4f,%3.4f,%3.4f, %3.1f,%3.2f,%d,%f\r",time,Yaw,Roll,Pitch,GyroX,GyroY,GyroZ,AccelX,AccelY,AccelZ,GPScourse,GPSspeed,GPSlat,GPSlong);                             
                        flag=0; // reset LogData interrupt flag
                        pc_led = !pc_led;  // Lights LED1 when transmitting to PC screen   
                    }    // end if(flag=1) loop
                 
                    if(enable==0) {
                    break;     // breaks while loop if enable switched off
                    }
            } // end while(!pc.readable()) loop            
            pc.printf(" Done. \n");     // prints 'done when logging is finished/enable switched off
            log_led=0;                  // turns off LED logging is finished/enable switched off
            wait(0.5);                  // debug wait for pc.printf      
            fclose(fp);                 // closes log file
            pc.getc();                  // Clear character from buffer
            
            while (!pc.readable()) {  // Wait for a key press to restart logging
            };
            pc.getc();            // Clear character from buffer
            FileNo++;             // Increment file number     
    } // end while(1) loop
 
} // end main() program


/* DEUBBING NOTES
   opening a file BEFORE calling interrupts                         OK
   opening a file and print to it BEFORE calling interrupts         NOT OK (stops rest of program)
   open a (local) file and print to it AFTER calling interrupts     NOT OK (stops rest of program)
   open a (sd) file and print to it AFTER calling interrupts        OK 
*/